From a6b3523dba4eff2067800daf1164d08f41fb4cc4 Mon Sep 17 00:00:00 2001
From: Andrew Allen <drewbitllama@gmail.com>
Date: Mon, 20 Nov 2017 17:55:56 -0800
Subject: [PATCH] Resolve align() issues in Projection API.

---
 src/mapping_matrix.c          | 14 ++++-----
 src/mapping_matrix.h          |  2 +-
 src/opus_projection_decoder.c |  8 +++---
 src/opus_projection_encoder.c | 67 +++++++++++++++++++------------------------
 4 files changed, 42 insertions(+), 49 deletions(-)

diff --git a/src/mapping_matrix.c b/src/mapping_matrix.c
index 5268bc6..c72f5bc 100644
--- a/src/mapping_matrix.c
+++ b/src/mapping_matrix.c
@@ -37,16 +37,16 @@
 
 #ifdef ENABLE_EXPERIMENTAL_AMBISONICS
 
-#define MATRIX_INDEX(nb_rows, row, col) (nb_rows * col + row)
+#define MATRIX_INDEX(nb_rows, row, col) ((opus_int32)nb_rows * col + row)
 
-int mapping_matrix_get_size(int rows, int cols)
+opus_int32 mapping_matrix_get_size(int rows, int cols)
 {
-  return align(sizeof(MappingMatrix)) + rows * cols * sizeof(opus_int16);
+  return align(sizeof(MappingMatrix)) + align(rows * cols * sizeof(opus_int16));
 }
 
 opus_int16 *mapping_matrix_get_data(const MappingMatrix *matrix)
 {
-  return (opus_int16*)(void *)(matrix + align(sizeof(MappingMatrix)));
+  return (opus_int16*)((char*)matrix + align(sizeof(MappingMatrix)));
 }
 
 void mapping_matrix_init(MappingMatrix * const matrix,
@@ -58,7 +58,7 @@ void mapping_matrix_init(MappingMatrix * const matrix,
 #if !defined(ENABLE_ASSERTIONS)
   (void)data_size;
 #endif
-  celt_assert((opus_uint32)data_size == rows * cols * sizeof(opus_int16));
+  celt_assert(align(data_size) == align(rows * cols * sizeof(opus_int16)));
 
   matrix->rows = rows;
   matrix->cols = cols;
@@ -128,10 +128,10 @@ void mapping_matrix_multiply_short(const MappingMatrix *matrix,
       for (col = 0; col < input_rows; col++)
       {
         tmp +=
-          (matrix_data[MATRIX_INDEX(matrix->rows, row, col)] *
+          ((opus_int32)matrix_data[MATRIX_INDEX(matrix->rows, row, col)] *
           input[MATRIX_INDEX(input_rows, col, i)]) >> 8;
       }
-      output[MATRIX_INDEX(output_rows, row, i)] = (tmp + 64)>>7;
+      output[MATRIX_INDEX(output_rows, row, i)] = (opus_int16)((tmp + 64)>>7);
     }
   }
 }
diff --git a/src/mapping_matrix.h b/src/mapping_matrix.h
index 8fe82ea..e8e4bd6 100644
--- a/src/mapping_matrix.h
+++ b/src/mapping_matrix.h
@@ -50,7 +50,7 @@ typedef struct MappingMatrix
     /* Matrix cell data goes here using col-wise ordering. */
 } MappingMatrix;
 
-int mapping_matrix_get_size(int rows, int cols);
+opus_int32 mapping_matrix_get_size(int rows, int cols);
 
 opus_int16 *mapping_matrix_get_data(const MappingMatrix *matrix);
 
diff --git a/src/opus_projection_decoder.c b/src/opus_projection_decoder.c
index 0d1ee41..34858f9 100644
--- a/src/opus_projection_decoder.c
+++ b/src/opus_projection_decoder.c
@@ -42,7 +42,7 @@
 
 struct OpusProjectionDecoder
 {
-  int demixing_matrix_size_in_bytes;
+  opus_int32 demixing_matrix_size_in_bytes;
   /* Encoder states go here */
 };
 
@@ -69,7 +69,7 @@ opus_int32 opus_projection_decoder_get_size(int channels, int streams,
   if (!decoder_size)
     return 0;
 
-  return align(sizeof(OpusProjectionDecoder) + matrix_size + decoder_size);
+  return align(sizeof(OpusProjectionDecoder)) + matrix_size + decoder_size;
 }
 
 int opus_projection_decoder_init(OpusProjectionDecoder *st, opus_int32 Fs,
@@ -93,7 +93,7 @@ int opus_projection_decoder_init(OpusProjectionDecoder *st, opus_int32 Fs,
   }
 
   /* Convert demixing matrix input into internal format. */
-  ALLOC(buf, demixing_matrix_size, opus_int16);
+  ALLOC(buf, demixing_matrix_size / sizeof(opus_int16), opus_int16);
   for (i = 0; i < nb_input_streams * channels; i++)
   {
     int s = demixing_matrix[2*i + 1] << 8 | demixing_matrix[2*i];
@@ -102,7 +102,7 @@ int opus_projection_decoder_init(OpusProjectionDecoder *st, opus_int32 Fs,
   }
 
   /* Assign demixing matrix. */
-  st->demixing_matrix_size_in_bytes = expected_matrix_size;
+  st->demixing_matrix_size_in_bytes = mapping_matrix_get_size(channels, nb_input_streams);
   mapping_matrix_init(get_demixing_matrix(st), channels, nb_input_streams, 0,
     buf, demixing_matrix_size);
 
diff --git a/src/opus_projection_encoder.c b/src/opus_projection_encoder.c
index a2c2f35..99c9e29 100644
--- a/src/opus_projection_encoder.c
+++ b/src/opus_projection_encoder.c
@@ -42,39 +42,38 @@
 
 struct OpusProjectionEncoder
 {
-  int mixing_matrix_size_in_bytes;
-  int demixing_matrix_size_in_bytes;
+  opus_int32 mixing_matrix_size_in_bytes;
+  opus_int32 demixing_matrix_size_in_bytes;
   /* Encoder states go here */
 };
 
-static int get_order_plus_one_from_channels(int channels, int *order_plus_one)
+static int validate_ambisonics_input(int channels, int *ambisonic_order)
 {
-  int order_plus_one_;
+  int order_plus_one;
   int acn_channels;
   int nondiegetic_channels;
 
   /* Allowed numbers of channels:
    * (1 + n)^2 + 2j, for n = 0...14 and j = 0 or 1.
    */
-  order_plus_one_ = isqrt32(channels);
-  acn_channels = order_plus_one_ * order_plus_one_;
+  order_plus_one = isqrt32(channels);
+  acn_channels = order_plus_one * order_plus_one;
   nondiegetic_channels = channels - acn_channels;
-  if (order_plus_one)
-    *order_plus_one = order_plus_one_;
+  if (ambisonic_order)
+    *ambisonic_order = order_plus_one - 1;
 
-  if (order_plus_one_ < 1 || order_plus_one_ > 15 ||
+  if (order_plus_one < 1 || order_plus_one > 15 ||
       (nondiegetic_channels != 0 && nondiegetic_channels != 2))
     return OPUS_BAD_ARG;
   return OPUS_OK;
 }
 
 static int get_streams_from_channels(int channels, int mapping_family,
-                                     int *streams, int *coupled_streams,
-                                     int *order_plus_one)
+                                     int *streams, int *coupled_streams)
 {
   if (mapping_family == 253)
   {
-    if (get_order_plus_one_from_channels(channels, order_plus_one) != OPUS_OK)
+    if (validate_ambisonics_input(channels, NULL) != OPUS_OK)
       return OPUS_BAD_ARG;
     if (streams)
       *streams = (channels + 1) / 2;
@@ -107,26 +106,18 @@ opus_int32 opus_projection_ambisonics_encoder_get_size(int channels,
 {
   int nb_streams;
   int nb_coupled_streams;
-  int order_plus_one;
-  int matrix_rows;
   opus_int32 matrix_size;
   opus_int32 encoder_size;
   int ret;
 
-  ret = get_streams_from_channels(channels, mapping_family, &nb_streams,
-                                  &nb_coupled_streams, &order_plus_one);
+  ret = get_streams_from_channels(channels, mapping_family, &nb_streams, &nb_coupled_streams);
   if (ret != OPUS_OK)
-  {
     return 0;
-  }
-
-  matrix_rows = order_plus_one * order_plus_one + 2;
-  matrix_size = mapping_matrix_get_size(matrix_rows, matrix_rows);
-  encoder_size =
-      opus_multistream_encoder_get_size(nb_streams, nb_coupled_streams);
+  matrix_size = mapping_matrix_get_size(channels, nb_streams + nb_coupled_streams);
+  encoder_size = opus_multistream_encoder_get_size(nb_streams, nb_coupled_streams);
   if (!encoder_size)
     return 0;
-  return align(sizeof(OpusProjectionEncoder) + matrix_size + matrix_size + encoder_size);
+  return align(sizeof(OpusProjectionEncoder)) + matrix_size + matrix_size + encoder_size;
 }
 
 int opus_projection_ambisonics_encoder_init(OpusProjectionEncoder *st, opus_int32 Fs,
@@ -144,7 +135,7 @@ int opus_projection_ambisonics_encoder_init(OpusProjectionEncoder *st, opus_int3
   unsigned char mapping[255];
 
   if (get_streams_from_channels(channels, mapping_family,
-                                &nb_streams, &nb_coupled_streams, NULL)
+                                &nb_streams, &nb_coupled_streams)
       != OPUS_OK)
     return OPUS_BAD_ARG;
 
@@ -156,55 +147,57 @@ int opus_projection_ambisonics_encoder_init(OpusProjectionEncoder *st, opus_int3
 
   if (mapping_family == 253)
   {
-    int order_plus_one;
-    if (get_order_plus_one_from_channels(channels, &order_plus_one) != OPUS_OK)
+    int ambisonic_order;
+    if (validate_ambisonics_input(channels, &ambisonic_order) != OPUS_OK)
       return OPUS_BAD_ARG;
 
     /* Assign mixing matrix based on available pre-computed matrices. */
     mixing_matrix = get_mixing_matrix(st);
-    if (order_plus_one == 2)
+    if (ambisonic_order == 1)
     {
       mapping_matrix_init(mixing_matrix, mapping_matrix_foa_mixing.rows,
         mapping_matrix_foa_mixing.cols, mapping_matrix_foa_mixing.gain,
         mapping_matrix_foa_mixing_data, 36 * sizeof(opus_int16));
     }
-    else if (order_plus_one == 3)
+    else if (ambisonic_order == 2)
     {
       mapping_matrix_init(mixing_matrix, mapping_matrix_soa_mixing.rows,
         mapping_matrix_soa_mixing.cols, mapping_matrix_soa_mixing.gain,
         mapping_matrix_soa_mixing_data, 121 * sizeof(opus_int16));
     }
-    else if (order_plus_one == 4)
+    else if (ambisonic_order == 3)
     {
       mapping_matrix_init(mixing_matrix, mapping_matrix_toa_mixing.rows,
         mapping_matrix_toa_mixing.cols, mapping_matrix_toa_mixing.gain,
         mapping_matrix_toa_mixing_data, 324 * sizeof(opus_int16));
     }
-    st->mixing_matrix_size_in_bytes = mapping_matrix_get_size(
-      mixing_matrix->rows, mixing_matrix->cols);
+    else
+    {
+      return OPUS_BAD_ARG;
+    }
+    st->mixing_matrix_size_in_bytes = mapping_matrix_get_size(mixing_matrix->rows, mixing_matrix->cols);
 
     /* Assign demixing matrix based on available pre-computed matrices. */
     demixing_matrix = get_demixing_matrix(st);
-    if (order_plus_one == 2)
+    if (ambisonic_order == 1)
     {
       mapping_matrix_init(demixing_matrix, mapping_matrix_foa_demixing.rows,
         mapping_matrix_foa_demixing.cols, mapping_matrix_foa_demixing.gain,
         mapping_matrix_foa_demixing_data, 36 * sizeof(opus_int16));
     }
-    else if (order_plus_one == 3)
+    else if (ambisonic_order == 2)
     {
       mapping_matrix_init(demixing_matrix, mapping_matrix_soa_demixing.rows,
         mapping_matrix_soa_demixing.cols, mapping_matrix_soa_demixing.gain,
         mapping_matrix_soa_demixing_data, 121 * sizeof(opus_int16));
     }
-    else if (order_plus_one == 4)
+    else if (ambisonic_order == 3)
     {
       mapping_matrix_init(demixing_matrix, mapping_matrix_toa_demixing.rows,
         mapping_matrix_toa_demixing.cols, mapping_matrix_toa_demixing.gain,
         mapping_matrix_toa_demixing_data, 324 * sizeof(opus_int16));
     }
-    st->demixing_matrix_size_in_bytes = mapping_matrix_get_size(
-      demixing_matrix->rows, demixing_matrix->cols);
+    st->demixing_matrix_size_in_bytes = mapping_matrix_get_size(demixing_matrix->rows, demixing_matrix->cols);
   }
   else
     return OPUS_UNIMPLEMENTED;
-- 
2.8.1.windows.1

